------------------------------------------------------------
revno: 9818
revision-id: squid3@treenet.co.nz-20091130104901-s1z3xt1qzaf0tuca
parent: squid3@treenet.co.nz-20091130104326-sx03kjpv3poat36v
committer: Amos Jeffries <squid3@treenet.co.nz>
branch nick: SQUID_3_1
timestamp: Mon 2009-11-30 23:49:01 +1300
message:
  Author: Henrik Nordstrom <henrik@henriknordstrom.net>
  Bug 2155: Assertion failures on malformed Content-Range response headers
  
  A number of conditions with malformed Content-Range headers were
  not trapped properly triggering odd conditions in the code.
------------------------------------------------------------
# Bazaar merge directive format 2 (Bazaar 0.90)
# revision_id: squid3@treenet.co.nz-20091130104901-s1z3xt1qzaf0tuca
# target_branch: http://www.squid-cache.org/bzr/squid3/trunk/
# testament_sha1: d5feb1be59151b37d481a7168dac23169637b5a5
# timestamp: 2009-11-30 10:51:43 +0000
# source_branch: http://www.squid-cache.org/bzr/squid3/branches\
#   /SQUID_3_1
# base_revision_id: squid3@treenet.co.nz-20091130104326-\
#   sx03kjpv3poat36v
# 
# Begin patch
=== modified file 'src/HttpHdrContRange.cc'
--- src/HttpHdrContRange.cc	2009-02-01 10:09:23 +0000
+++ src/HttpHdrContRange.cc	2009-11-30 10:49:01 +0000
@@ -84,22 +84,34 @@
     if (!httpHeaderParseOffset(field, &spec->offset))
         return 0;
 
+    /* Additional check for BUG2155 - there MUST BE first-byte-pos and it MUST be positive*/
+    if (spec->offset < 0) {
+        debugs(68, 2, "invalid (no first-byte-pos or it is negative) resp-range-spec near: '" << field << "'");
+        return 0;
+    }
+
     p++;
 
     /* do we have last-pos ? */
-    if (p - field < flen) {
-        int64_t last_pos;
-
-        if (!httpHeaderParseOffset(p, &last_pos))
-            return 0;
-
-        spec->length = size_diff(last_pos + 1, spec->offset);
-        /* Ensure typecast is safe */
-        assert (spec->length >= 0);
-    }
+    if (p - field >= flen) {
+        debugs(68, 2, "invalid (no last-byte-pos) resp-range-spec near: '" << field << "'");
+        return 0;
+    }
+
+    int64_t last_pos;
+
+    if (!httpHeaderParseOffset(p, &last_pos))
+        return 0;
+
+    if (last_pos < spec->offset) {
+        debugs(68, 2, "invalid (negative last-byte-pos) resp-range-spec near: '" << field << "'");
+        return 0;
+    }
+
+    spec->length = size_diff(last_pos + 1, spec->offset);
 
     /* we managed to parse, check if the result makes sence */
-    if (known_spec(spec->length) && spec->length == 0) {
+    if (spec->length <= 0) {
         debugs(68, 2, "invalid range (" << spec->offset << " += " <<
                (long int) spec->length << ") in resp-range-spec near: '" << field << "'");
         return 0;
@@ -176,6 +188,14 @@
         range->elength = range_spec_unknown;
     else if (!httpHeaderParseOffset(p, &range->elength))
         return 0;
+    else if (range->elength <= 0) {
+        /* Additional paranoidal check for BUG2155 - entity-length MUST be > 0 */
+        debugs(68, 2, "invalid (entity-length is negative) content-range-spec near: '" << str << "'");
+        return 0;
+    } else if (known_spec(range->spec.length) && range->elength < (range->spec.offset + range->spec.length)) {
+        debugs(68, 2, "invalid (range is outside entity-length) content-range-spec near: '" << str << "'");
+        return 0;
+    }
 
     debugs(68, 8, "parsed content-range field: " <<
            (long int) range->spec.offset << "-" <<

